;-----------------------------------------------------------------------;
;                                                                       ;
; Alien Breed IV: Final Assault                                         ;
; Version 1.1.0                                                         ;
;                                                                       ;
; Copyright (c)2001-2012 James Vernon                                   ;
; jamesv82@live.com.au                                                  ;
; http://www.jvti.org                                                   ;
;                                                                       ;
; You are allowed to use parts of this code as long as you              ;
; give credit where it is due.                                          ;
;                                                                       ;
;-----------------------------------------------------------------------;

#define BREAK() ld a,$F7 \ call directIn \ bit 0,a \ jr nz,$+8 \ call waitKey \ call waitKey

#include "ab4.h"                                ; All header information

#ifdef TI86
#include "ab4data.inc"
#endif

;------------------------------------------------
; Start of Program
;------------------------------------------------
start:
;        bcall(_homeup)
;        ld      hl,endCode-start-12
;        bcall(_disphl)
;        call    waitKey
#ifdef TI86
        call    _flushallmenus
        call    _runindicoff
        ld      hl,dataFileName-1
        rst     20h
        rst     10h
        jp      c,quit
        call    _ex_ahl_bde
        call    _get_word_ahl
        call    _set_abs_src_addr
        ld      a,1
        ld      hl,dataFileStorage-$8000
        call    _set_abs_dest_addr
        xor     a
        ex      de,hl
        call    _set_mm_num_bytes
        call    _mm_ldir
        call    _ram_page_1                     ; TI-86 needs RAM page 1 loaded
#endif
#ifdef TI83
        res     statsvalid,(iy+statflags)
#endif
#ifndef TI86
        ld      hl,gbuf+768
        ld      de,system
        ld      bc,256
        ldir
#endif
#ifdef TI83
        call    _memfree
        ld      de,MEM_NEEDED
        sbc     hl,de
        jp      c,quit
#endif
#ifdef PLUS
        ld      hl,MEM_NEEDED
        bcall(_enoughMem)
        jp      c,quit
#endif
#ifndef TI86
        call    checkTempFile
        ld      hl,LEVEL_SIZE
        bcall(_createprog)
        inc     de
        inc     de
        ld      (tmpProg),de
#endif
        set     apdable,(iy+apdflags)
        set     textwrite,(iy+sgrflags)
        xor     a
        ld      (menuItem),a

startVAT:
#ifdef TI86
        ld      hl,$BFFF
#else
        ld      hl,(vat)
#endif
        ld      (vatPtr),hl
        xor     a
        ld      (numLevelsets),a
nextLevelset:
        ld      hl,(vatPtr)
levelsetSearch:
        ld      ix,searchString
        call    ionDetect
        jp      z,levelsetFound
levelsetNotFound:
        ld      a,(numLevelsets)
        or      a
        jp      z,quit
        jr      startVAT

menuLoop:
        bcall(_grbufclr)
        ld      hl,titleData
        ld      de,gbuf
        GETTEMP(ix)
        call    huffExtr

        ld      hl,levelsetName
#ifdef TI86
        ld      de,30*256+88
#else
        ld      de,30*256+72
#endif
        call    vPuts                           ; Show Levelset name

#ifdef TI86
        ld      hl,44*256+88
#else
        ld      hl,44*256+72
#endif
        ld      (pencol),hl
        ld      hl,(hiScore)
        call    showHL                          ; Show HiScore

        ld      hl,(menuItem)
#ifdef TI86
        ld      h,7*16
#else
        ld      h,7*12
#endif
        bcall(_htimesl)
#ifdef TI86
        ld      de,gbuf+(28*16)+3
#else
        ld      de,gbuf+(28*12)+1
#endif
        add     hl,de
        ld      bc,7*256+7
        call    invertBox
        call    ionFastCopy
        call    waitKey
        ld      hl,menuItem
        cp      GK_UP
        jr      z,menuUp
        cp      GK_DOWN
        jr      z,menuDown
        cp      GK_2ND
        jr      nz,menuLoop

        ld      a,(hl)
        or      a
        jr      z,newGame
        dec     a
        jp      z,getPasscode
        dec     a
        jr      z,nextLevelset

quit:
        res     textwrite,(iy+sgrflags)
#ifndef TI86
        ld      hl,system
        ld      de,gbuf+768
        ld      bc,256
        ldir
#endif
#ifdef TI86
        call    _clrscrnfull
        call    _homeup
        set     graphdraw,(iy+graphflags)
#endif
#ifdef TI83
        set     statsvalid,(iy+statflags)
#endif
#ifndef TI86
checkTempFile:
        ld      hl,tempName
        MOVE9()
        bcall(_chkfindsym)
        bcallnc(_delvar)
#endif
        ret

menuUp:
        ld      a,(hl)
        or      a
        jp      z,menuLoop
        dec     (hl)
        jp      menuLoop
menuDown:
        ld      a,(hl)
        cp      3
        jp      z,menuLoop
        inc     (hl)
        jp      menuLoop

levelsetFound:
        ld      (vatPtr),de
        ld      de,levelsetName
        ld      bc,3
        ldir
        ld      (de),a
        ld      a,(hl)                          ; A = Number of levels in this Levelset
        ld      (numLevels),a
        inc     hl
        ld      e,(hl)
        inc     hl
        ld      d,(hl)                          ; DE = HiScore for this Levelset
        inc     hl
        ld      (passcodePtr),hl
        ld      (hiScore),de
        ld      de,7*6
        add     hl,de
        ld      (dataPtr),hl
        ld      hl,numLevelsets
        inc     (hl)
        jp      menuLoop

newGame:
        xor     a
        ld      (level),a
startGame:
        sbc     hl,hl
        ld      (credits),hl
        ld      (score),hl
        ld      (keyPasses),a
        ld      (playerDir),a
        ld      (radarScanner),a
        inc     a
        ld      (weapon),a
        ld      hl,weaponTable
        ld      (hl),a
        inc     hl
        xor     a
        ld      (hl),a
        inc     hl
        ld      (hl),a
        inc     hl
        ld      (hl),a
        inc     hl
        ld      (hl),a
        ld      hl,50
        ld      (ammo),hl
        ld      a,3
        ld      (lives),a
        ld      a,100
        ld      (health),a
newLevel:
        xor     a
        ld      (dead),a
        ld      (hurt),a
        ld      (invincible),a
        ld      (queen),a
        ld      (fireDoor+1),a
        ld      (securityGuardCnt),a
        ld      (securityGuardMissile),a
        ld      hl,keyTable
        ld      (hl),a
        inc     hl
        ld      (hl),a
        inc     hl      
        ld      (hl),a
        ld      a,(x)
        inc     a
        inc     a
        inc     a
        ld      hl,(y)
        inc     l
        inc     l
        inc     l
        ld      h,a
        ld      (playerCenter),hl
        call    clearAlienHoleTable
        call    clearAnimTable
        call    clearBulletTable
        call    clearEnemyTable
        call    clearSecurityGuardTable
        ld      a,(level)
        call    loadLevel
#include "main.asm"                             ; Main Loop

levelFinished:
        ld      de,500
        call    addScore
        ld      hl,level
        inc     (hl)
        ld      a,(numLevels)
        cp      (hl)
        jp      nz,newLevel

levelsetFinished:
        bcall(_grbufclr)
        bcall(_homeup)
        ld      hl,strLevelsetFinished
        ld      de,20*256+10
        call    vPuts
        ld      de,28*256+33
        call    vPuts
        ld      hl,strPressAnyKey
        ld      de,57*256+25
        call    vPuts
#ifdef TI86
        call    videoCenterCopy
        ld      hl,$FC00+(57*16)
        ld      bc,7*256+16
        call    invertBox
        ld      hl,strCongratulations
        bcall(_puts)
        ld      hl,$FC00
        ld      bc,8*256+16
        call    invertBox
#else
        ld      hl,gbuf+(57*12)
        ld      bc,7*256+12
        call    invertBox
        call    ionFastCopy
        ld      hl,strCongratulations
        set     textinverse,(iy+textflags)
        bcall(_puts)
        res     textinverse,(iy+textflags)
#endif
        call    waitKeyA

gameOver:
        bcall(_clrscrnfull)
#ifdef TI86
        ld      hl,5*256+2
#else
        ld      hl,3*256+2
#endif
        ld      (currow),hl
        ld      hl,strGameOver
        bcall(_puts)
        ld      hl,(hiScore)
        ld      de,(score)
        or      a
        sbc     hl,de
        jr      nc,noNewHiScore
        ld      (hiScore),de
#ifdef TI86
        push    de
        ld      hl,(realFilePtr)
        ld      a,(realFilePtr+2)
        ld      de,10
        add     hl,de
        adc     a,0
        pop     de
        call    _set_word_ahl
        call    _ram_page_1
#else
        ld      hl,(passcodePtr)
        dec     hl
        ld      (hl),d
        dec     hl
        ld      (hl),e
#endif
#ifdef TI86
        ld      hl,4*256+5
#else
        ld      hl,1*256+5
#endif
        ld      (currow),hl
        ld      hl,strNewHiScore
        bcall(_puts)
noNewHiScore:
        call    waitKeyA
        jp      menuLoop

;------------------------------------------------
; Other routine files
;------------------------------------------------
#include "anim.asm"                             ; Animation Routines
#include "bullets.asm"                          ; Bullet Routines
#include "collide.asm"                          ; Collision Detection Routines
#include "damage.asm"                           ; Damage Checking Routines
#include "enemy.asm"                            ; Enemy Routines
#include "huffextr.h"                           ; Huffman Extraction
#include "intex.asm"                            ; Intex Console Routines
#include "loadlvl.asm"                          ; Level Loading Routines
#include "misc.asm"                             ; Miscellaneous Routines
#include "passcode.asm"                         ; Passcode Handling Routines
#include "pause.asm"                            ; Paused Routines
#include "player.asm"                           ; Player Routines
#include "queen.asm"                            ; Queen Alien Routines
#include "sprite.asm"                           ; Sprite Routines
#ifdef TI86
#include "scroll16.asm"                         ; Wide Screen Scrolling Routines
#else
#include "scroll12.asm"                         ; Small Screen Scrolling Routines
#endif

endCode:

#ifndef TI86
#include "data.asm"                             ; All strings and other data
#else
dataFileName:
.db     7, "ab4data"                            ; Data file name
#endif

bulletMoveTable:
.dw     moveBulletUp, moveBulletDown, moveBulletLeft, moveBulletRight

intexJumpTable:
.dw     intexMainJumpTable, intexWeaponsJumpTable, intexMiscJumpTable

intexMainJumpTable:
.dw     intexWeaponsMenu, intexMiscMenu, intexMap, intexExit

intexWeaponsJumpTable:
.dw     intexBuyTwinMissile, intexBuyFlameArc, intexBuyPlasmaRifle, intexBuyRocketLauncher, intexMainMenu

intexMiscJumpTable
.dw     intexBuyKeyPack, intexBuyAmmoClip, intexBuyFirstAid, intexBuyRadarScanner, intexBuyExtraLife, intexMainMenu

#ifdef TI83
ti83Vars:
.db     0,0,0,0,0,0,0,0,0,0
.db     0,0,0,0,0,0,0,0,0,0
.db     0,0,0,0,0,0,0,0,0,0
.db     0,0,0,0,0,0,0,0,0,0
.db     0,0,0,0,0,0,0,0,0,0
.db     0,0,0,0,0,0,0,0,0,0
.db     0,0,0,0,0,0,0,0,0,0
.db     0,0,0,0,0,0

#endif

#include "vars.asm"

endProgram:

.end
