;-----------------------------------------------------------;
;                                                           ;
; Alien Breed IV: Final Assault                             ;
; Animation Routines                                        ;
;                                                           ;
;-----------------------------------------------------------;

;------------------------------------------------
; clearAnimTable - Clear animTable
;
; Input:    None
; Output:   None
;------------------------------------------------
clearAnimTable:
        ld      hl,animTable
        ld      de,animTable+1
        ld      bc,NUM_ANIMS*4-1
        ld      (hl),0
        ldir
        ret

;------------------------------------------------
; newAnim - Create a new animation
;
; Input:    B = Ini Counter Value
;           C = Type
;           H = X
;           L = Y
; Output:   None
;------------------------------------------------
newAnim:
        push    ix
        push    bc
        ld      ix,animTable
        ld      b,NUM_ANIMS
        ld      de,4
findEmptyAnim:
        ld      a,(ix)
        or      a
        jr      z,foundEmptyAnim
        add     ix,de
        djnz    findEmptyAnim
        pop     bc
        jr      leaveNewAnim
foundEmptyAnim:
        pop     bc
        ld      (ix),c
        ld      (ix+1),b
        ld      (ix+2),h
        ld      (ix+3),l
leaveNewAnim:
        pop     ix
        ret

;------------------------------------------------
; updateAnims - Update all active animations
;
; Input:    None
; Output:   None
;------------------------------------------------
updateAnims:
        ld      ix,animTable
        ld      b,NUM_ANIMS
        ld      de,4
updateAnimsLoop:
        ld      a,(ix)
        or      a
        jr      z,endUpdateAnimsLoop
        dec     (ix+1)
        jr      nz,endUpdateAnimsLoop
        ld      (ix),0
endUpdateAnimsLoop:
        add     ix,de
        djnz    updateAnimsLoop
        ret

;------------------------------------------------
; drawAnims - Draw animations
;
; Input:    None
; Output:   None
;------------------------------------------------
drawAnims:
        ld      ix,animTable
        ld      b,NUM_ANIMS
drawAnimsLoop:
        ld      a,(ix)
        or      a
        jr      z,endDrawAnimsLoop
        push    bc
        dec     a
        add     a,a
        ld      l,a
        ld      h,0
        ld      de,animSpriteTable
        add     hl,de
        ld      e,(hl)
        inc     hl
        ld      d,(hl)
        ld      l,(ix+1)
        dec     l
        ld      h,0
        TIMES8(hl)
        add     hl,de
        ex      de,hl
        ld      a,(ix+2)
        ld      hl,scrX
        sub     (hl)
        ld      b,a
        inc     hl
        ld      a,(ix+3)
        sub     (hl)
        ld      c,a
        ex      de,hl
        push    ix
        call    putClippedSprite
        pop     ix
        pop     bc
endDrawAnimsLoop:
        ld      de,4
        add     ix,de
        djnz    drawAnimsLoop
        ret

.end
