;-----------------------------------------------------------;
;                                                           ;
; Alien Breed IV: Final Assault                             ;
; Bullet Routines                                           ;
;                                                           ;
;-----------------------------------------------------------;

;------------------------------------------------
; clearBulletTable - Clear bulletTable
;
; Input:    None
; Output:   None
;------------------------------------------------
clearBulletTable:
        ld      hl,bulletTable
        ld      de,bulletTable+1
        ld      bc,NUM_BULLETS*4-1
        ld      (hl),0
        ldir
        ret

;------------------------------------------------
; shoot - Try to create a new bullet
;
; Input:    None
; Output:   None
;------------------------------------------------
shoot:
        ld      hl,frame
        bit     0,(hl)
        ret     nz
        ld      hl,(ammo)
        ld      a,l
        or      h
        ret     z
        ld      ix,bulletTable
        ld      b,NUM_BULLETS
        ld      de,4
findEmptyBullet:
        ld      a,(ix)
        or      a
        jr      z,foundEmptyBullet
        add     ix,de
        djnz    findEmptyBullet
        ret
foundEmptyBullet:
        dec     hl
        ld      (ammo),hl
        ld      hl,playerDir
        ld      a,(weapon)
        ld      bc,x
makeNewBullet:
        ld      (ix),a
        dec     a
        TIMES8(a)
        ld      d,a
        ld      a,(hl)
        ld      (ix+1),a
        add     a,a
        add     a,d
        ld      l,a
        ld      h,0
        ld      de,bulletOffsetTable
        add     hl,de
        ld      a,(bc)
        add     a,(hl)
        ld      (ix+2),a
        inc     hl
        inc     bc
        ld      a,(bc)
        add     a,(hl)
        ld      (ix+3),a
        ret

;------------------------------------------------
; drawBullets - Draw bullets
;
; Input:    None
; Output:   None
;------------------------------------------------
drawBullets:
        ld      ix,bulletTable
        ld      b,NUM_BULLETS
drawBulletsLoop:
        push    bc
        ld      a,(ix)
        or      a
        jr      z,endDrawBulletsLoop
        dec     a
        TIMES8(a)
        ld      c,a
        ld      b,0
        ld      h,b
        ld      l,(ix+1)
        add     hl,hl
        add     hl,hl
        ld      e,l
        ld      d,h
        add     hl,hl
        add     hl,hl
        add     hl,de
        add     hl,hl
        ld      de,sprBullets
        add     hl,de
        add     hl,bc
        ex      de,hl
        ld      hl,scrX
        ld      a,(ix+2)
        sub     (hl)
        ld      b,a
        inc     hl
        ld      a,(ix+3)
        sub     (hl)
        ld      c,a
        ex      de,hl
        push    ix
        call    putClippedSprite
        pop     ix
endDrawBulletsLoop:
        pop     bc
        ld      de,4
        add     ix,de
        djnz    drawBulletsLoop
        ret

;------------------------------------------------
; moveBullets - Move bullets
;
; Input:    None
; Output:   None
;------------------------------------------------
moveBullets:
        ld      ix,bulletTable
        ld      b,NUM_BULLETS
moveBulletsLoop:
        push    bc
        ld      a,(ix)
        or      a
        jr      z,endMoveBulletsLoop
        ld      l,(ix+1)
        ld      h,0
        add     hl,hl
        ld      de,bulletMoveTable
        add     hl,de
        bcall(_ldhlind)
        ld      de,endMoveBulletsLoop
        push    de
        jp      (hl)
endMoveBulletsLoop:
        pop     bc
        ld      de,4
        add     ix,de
        djnz    moveBulletsLoop
        ret
moveBulletUp:
        ld      a,(ix+3)
        or      a
        jr      z,removeBullet
        dec     a
        jr      z,removeBullet
        dec     a
        ld      (ix+3),a
        jr      moveBulletCheck
moveBulletDown:
        ld      a,(height)
        TIMES8(a)
        cp      (ix+3)
        jr      z,removeBullet
        inc     (ix+3)
        cp      (ix+3)
        jr      z,removeBullet
        inc     (ix+3)
        jr      moveBulletCheck
moveBulletLeft:
        ld      a,(ix+2)
        or      a
        jr      z,removeBullet
        dec     a
        jr      z,removeBullet
        dec     a
        ld      (ix+2),a
        jr      moveBulletCheck
moveBulletRight:
        ld      a,(width)
        TIMES8(a)
        cp      (ix+2)
        jr      z,removeBullet
        inc     (ix+2)
        cp      (ix+2)
        jr      z,removeBullet
        inc     (ix+2)
        jr      moveBulletCheck
removeBullet:
        ld      (ix),0
        ret
shotEgg:
        call    doShootingObjectStuff
        jr      removeBullet
shotPowerDome:
        call    doShootingObjectStuff
        call    lowerObjectives
        jr      removeBullet
doShootingObjectStuff:
        push    ix
        inc     a
        call    putTile
        ld      de,30
        call    addScore
        pop     ix
        ret
moveBulletCheck:
        ld      a,(ix)
        dec     a
        TIMES8(a)
        ld      l,a
        ld      a,(ix+1)
        add     a,a
        add     a,l
        ld      l,a
        ld      h,0
        ld      de,bulletCenterTable
        add     hl,de
        ld      a,(ix+2)
        add     a,(hl)
        ld      b,a
        inc     hl
        ld      a,(ix+3)
        add     a,(hl)
        ld      l,a
        ld      a,b
        call    getTile
        cp      POWER_DOME_TILE
        jr      z,shotPowerDome
        cp      EGG_TILE
        jr      z,shotEgg
        cp      SOLID_TILE
        jr      nc,removeBullet
        ret

.end
