;-----------------------------------------------------------;
;                                                           ;
; Alien Breed IV: Final Assault                             ;
; Damage Checking Routines                                  ;
;                                                           ;
;-----------------------------------------------------------;

;------------------------------------------------
; checkBulletEnemyCollisions - Check for collisions between bullets and enemies
;
; Input:    None
; Output:   None
;------------------------------------------------
checkBulletEnemyCollisions:
        ld      b,NUM_BULLETS
        di
        push    iy
        ld      iy,bulletTable
beCollisionsLoop:
        push    bc
        ld      a,(iy)
        or      a
        jp      z,endBECollisionsLoop
        call    getBulletDetails
        ld      (_beDamage),a
        ld      ix,enemyTable
        ld      b,NUM_ENEMIES
beCollisionsLoop2:
        push    bc
        ld      a,(ix)
        or      a
        jr      z,endBECollisionsLoop2
        call    loadEnemyToCollideObject2
        call    checkCollision
        jr      nc,endBECollisionsLoop2
        ld      (iy),0
        ld      a,(ix+5)
        sub     $00
_beDamage       = $-1
        ld      (ix+5),a
        cp      1
        jp      p,endBECollisionsLoop2
        call    enemyDead
endBECollisionsLoop2:
        pop     bc
        ld      de,7
        add     ix,de
        djnz    beCollisionsLoop2
endBECollisionsLoop:
        pop     bc
        ld      de,4
        add     iy,de
        dec     b
        jp      nz,beCollisionsLoop
        pop     iy
        ei
        ret

;------------------------------------------------
; checkEnemyPlayerCollisions - Check for collisions between player and enemies
;
; Input:    None
; Output:   None
;------------------------------------------------
checkEnemyPlayerCollisions:
        ld      hl,collideObject1
        call    loadPlayerToCollideObject
        ld      ix,enemyTable
        ld      b,NUM_ENEMIES
epCollisionsLoop:
        push    bc
        ld      a,(ix)
        or      a
        jr      z,endEPCollisionsLoop
        call    loadEnemyToCollideObject2
        call    checkCollision
        jr      nc,endEPCollisionsLoop
        ld      a,(dead)
        or      a
        jr      nz,enemyNotHurtPlayer
        ld      a,(invincible)
        or      a
        jr      nz,enemyNotHurtPlayer
        ld      l,(ix)
        dec     l
        ld      h,0
        ld      de,enemyDamageTable
        add     hl,de
        ld      a,(health)
        sub     (hl)
        ld      (health),a
        ld      a,1
        ld      (hurt),a
enemyNotHurtPlayer:
        ld      (ix+5),0
        call    enemyDead
endEPCollisionsLoop:
        pop     bc
        ld      de,7
        add     ix,de
        djnz    epCollisionsLoop
        ret

;------------------------------------------------
; checkPlayerQueenCollisions - Chekc for collisions between the player and the Queen
;
; Input:    None
; Output:   None
;------------------------------------------------
checkPlayerQueenCollisions:
        ld      a,(dead)
        or      a
        ret     nz
        ld      a,(invincible)
        or      a
        ret     nz
        ld      hl,collideObject1
        call    loadPlayerToCollideObject
        call    loadQueenToCollideObject2
        call    checkCollision
        ret     nc
        ld      hl,health
        ld      a,(hl)
        sub     QUEEN_STRENGTH
        ld      (hl),a
        ret

;------------------------------------------------
; checkBulletQueenCollisions - Check for collisions between bullets and the Queen
;
; Input:    None
; Output:   None
;------------------------------------------------
checkBulletQueenCollisions:
        call    loadQueenToCollideObject2
        di
        push    iy
        ld      iy,bulletTable
        ld      b,NUM_BULLETS
bqCollisionsLoop:
        push    bc
        ld      a,(iy)
        or      a
        jr      z,endBQCollisionsLoop
        call    getBulletDetails
        ld      (_bqDamage),a
        call    checkCollision
        jr      nc,endBQCollisionsLoop
        ld      h,(iy+2)
        ld      l,(iy+3)
        ld      bc,4*256+1
        call    newAnim
        ld      (iy),0
        ld      hl,(queenHealth)
        ld      de,$0000
_bqDamage       = $-2
        or      a
        sbc     hl,de
        ld      (queenHealth),hl
        bit     7,h
        jp      nz,queenDead
endBQCollisionsLoop:
        pop     bc
        ld      de,4
        add     iy,de
        djnz    bqCollisionsLoop
        pop     iy
        ei
        ret

;------------------------------------------------
; checkPlayerSGMissileCollisions - Check for collision between player and Security Guard Missile
;
; Input:    None
; Output:   None
;------------------------------------------------
checkPlayerSGMissileCollisions:
        ld      a,(securityGuardMissile)
        or      a
        ret     z
        ld      hl,collideObject2
        call    loadPlayerToCollideObject
        di
        push    iy
        ld      iy,securityGuardMissile
        call    getBulletDetails
        pop     iy
        ei
        call    checkCollision
        ret     nc
        xor     a
        ld      (securityGuardMissile),a
        ld      hl,health
        ld      a,(hl)
        sub     5
        ld      (hl),a
        ld      a,1
        ld      (hurt),a
        ret

;------------------------------------------------
; loadPlayerToCollideObject - Load the player into to a collideObject
;
; Input:    None
; Output:   None
;------------------------------------------------
loadPlayerToCollideObject:
        ld      a,(x)
        ld      (hl),a
        inc     hl
        ld      (hl),8
        inc     hl
        ld      a,(y)
        ld      (hl),a
        inc     hl
        ld      (hl),8
        ret

;------------------------------------------------
; loadEnemyToCollideObject2 - Load an enemy into collideObject2
;
; Input:    IX => Enemy entry in enemyTable
; Output:   IX => Enemy entry in enemyTable
;------------------------------------------------
loadEnemyToCollideObject2:
        ld      a,(ix+2)
        ld      hl,collideObject2
        ld      (hl),a
        inc     hl
        ld      (hl),8
        inc     hl
        ld      a,(ix+3)
        ld      (hl),a
        inc     hl
        ld      (hl),8
        ret

;------------------------------------------------
; loadQueenToCollideObject2 - Load Queen info into collideObject2
;
; Input:    None
; Output:   None
;------------------------------------------------
loadQueenToCollideObject2:
        ld      ix,qX-2
        call    loadEnemyToCollideObject2
        ld      (hl),24
        dec     hl
        dec     hl
        ld      (hl),16
        ret

;------------------------------------------------
; getBulletDetails - Get bullet damage value and load it to collideObject1
;
; Input:    IY => Bullet entry in bulletTable
; Output:   A = Damage value
;           IY => Bullet entry in bulletTable
;------------------------------------------------
getBulletDetails:
        ld      l,(iy)
        ld      h,0
        dec     l
        TIMES8(hl)
        add     hl,hl
        ld      de,bulletSizeTable
        add     hl,de
        ld      a,(iy+1)
        add     a,a
        ld      e,a
        ld      d,0
        add     hl,de
        ex      de,hl
        ld      hl,collideObject1
        ld      a,(iy+2)
        ld      (hl),a
        inc     hl
        ld      a,(de)
        ld      (hl),a
        inc     de
        inc     hl
        ld      a,(iy+3)
        ld      (hl),a
        inc     hl
        ld      a,(de)
        ld      (hl),a
        ld      a,(iy)
        dec     a
        ld      e,a
        ld      d,0
        ld      hl,bulletDamageTable
        add     hl,de
        ld      a,(hl)
        ret

.end
