;-----------------------------------------------------------;
;                                                           ;
; Alien Breed IV: Final Assault                             ;
; Intex Console Routines                                    ;
;                                                           ;
;-----------------------------------------------------------;

;------------------------------------------------
; doIntexConsole - Execute Intex Console code
;
; Input:    None
; Output:   None
;------------------------------------------------
doIntexConsole:
        ld      hl,(playerDir)
        ld      h,0
        add     hl,hl
        ld      de,intexTable
        add     hl,de
        ld      a,(y)
        add     a,(hl)
        inc     hl
        ld      b,a
        ld      a,(x)
        add     a,(hl)
        ld      l,b
        call    getTile
        cp      INTEX_TILE
        ret     nz
        call    saveVideo

;------------------------------------------------
; intexMainMenu - Do Main Menu in Intex Network System
;
; Input:    None
; Output:   None
;------------------------------------------------
intexMainMenu:
        ld      hl,intexMainMenuData
        jp      doIntexMenu

;------------------------------------------------
; intexWeaponsMenu - Do Weapons Menu in Intex Network System
;
; Input:    None
; Output:   None
;------------------------------------------------
intexWeaponsMenu:
        ld      hl,intexWeaponsMenuData
        jp      doIntexMenu

;------------------------------------------------
; intexMiscMenu - Do Misc Menu in Intex Network System
;
; Input:    None
; Output:   None
;------------------------------------------------
intexMiscMenu:
        ld      hl,intexMiscMenuData

;------------------------------------------------
; doIntexMenu - Run a specified menu in the Intex Network System
;
; Input:    HL => Text data for menu
; Output:   None
;------------------------------------------------
doIntexMenu:
        push    hl
        bcall(_grbufclr)
        pop     hl
        ld      a,(hl)
        inc     hl
        ld      (intexOptions),a
        ld      b,(hl)
        inc     hl
writeIntexMenuText:
        push    bc
        ld      d,(hl)
        inc     hl
        ld      e,(hl)
        inc     hl
        call    vPuts
        pop     bc
        djnz    writeIntexMenuText
        ld      a,(intexOptions)
        cp      4
        jr      z,afterShowIntexCredits
        ld      hl,strIntexCredits
        ld      de,57*256+2
        call    vPuts
        ld      hl,(credits)
        call    showHL
afterShowIntexCredits:
        ld      hl,gbuf
        ld      bc,7*256+12
        call    invertBox
#ifdef TI86
        ld      hl,gbuf+(57*16)
#else
        ld      hl,gbuf+(57*12)
#endif
        ld      bc,7*256+12
        call    invertBox
        xor     a
        ld      (intexSelection),a
#ifdef TI86
        call    videoCenterCopy
#endif
intexMenuLoop:
        call    highlightIntexSelection
        call    ionFastCopy
        call    highlightIntexSelection
        call    waitKey
        ld      hl,intexSelection
        cp      GK_UP
        jr      z,intexUp
        cp      GK_DOWN
        jr      z,intexDown
        cp      GK_2ND
        jr      nz,intexMenuLoop
        ld      a,(intexOptions)
        sub     4
        add     a,a
        ld      e,a
        ld      d,0
        ld      hl,intexJumpTable
        add     hl,de
        ld      e,(hl)
        inc     hl
        ld      d,(hl)
        ld      hl,(intexSelection)
        ld      h,0
        add     hl,hl
        add     hl,de
        ld      a,(hl)
        inc     hl
        ld      h,(hl)
        ld      l,a
        jp      (hl)
intexUp:
        ld      a,(hl)
        or      a
        jr      z,intexMenuLoop
        dec     (hl)
        jr      intexMenuLoop
intexDown:
        ld      a,(intexOptions)
        dec     a
        cp      (hl)
        jr      z,intexMenuLoop
        inc     (hl)
        jr      intexMenuLoop

;------------------------------------------------
; highlightIntexSelection - Invert current selection in Intex Menu
;
; Input:    None
; Output:   None
;------------------------------------------------
highlightIntexSelection:
        ld      hl,(intexSelection)
#ifdef TI86
        ld      h,16*7
        bcall(_htimesl)
        ld      de,gbuf+(10*16)+2
#else
        ld      h,12*7
        bcall(_htimesl)
        ld      de,gbuf+(10*12)
#endif
        add     hl,de
        ld      bc,7*256+12
        jp      invertBox

;------------------------------------------------
; intexMap - Show map
;
; Input:    None
; Output:   None
;------------------------------------------------
intexMap:
        ld      hl,intexMainMenu
        push    hl
showMap:
        bcall(_grbufclr)
        xor     a
        ld      (_mapY),a
mapNextY:
        xor     a
        ld      (_mapX),a
        ld      a,(_mapY)
        sbc     hl,hl
        or      a
        jr      z,mapSkip
        ld      hl,(width)
        inc     l
        ld      h,a
        bcall(_htimesl)
mapSkip:
        GETTEMP(de)
        add     hl,de
mapNextX:
        push    hl
        ld      a,(hl)
        cp      SOLID_TILE
        jr      c,mapAfterPutPixels
        cp      NORMAL_DOOR
        jr      nc,mapAfterPutPixels
        ld      e,$00
_mapY           = $-1
        ld      d,$00
_mapX           = $-1
        sla     d
        sla     e
        push    de
        ld      a,d
        call    ionGetPixel
        or      (hl)
        ld      (hl),a
        pop     de
        inc     d
        push    de
        ld      a,d
        call    ionGetPixel
        or      (hl)
        ld      (hl),a
        pop     de
        inc     e
        push    de
        ld      a,d
        call    ionGetPixel
        or      (hl)
        ld      (hl),a
        pop     de
        dec     d
        ld      a,d
        call    ionGetPixel
        or      (hl)
        ld      (hl),a
mapAfterPutPixels:
        pop     hl
        inc     hl
        ld      a,(_mapX)
        inc     a
        ld      (_mapX),a
        ld      b,a
        ld      a,(width)
        inc     a
        cp      b
        jr      nz,mapNextX
        ld      a,(_mapY)
        inc     a
        ld      (_mapY),a
        ld      b,a
        ld      a,(height)
        inc     a
        cp      b
        jr      nz,mapNextY
        ld      a,(y)
        inc     a
        inc     a
        inc     a
        call    divAby8
        add     a,a
        ld      e,a
        ld      a,(x)
        inc     a
        inc     a
        inc     a
        call    divAby8
        add     a,a
        ld      d,a
        push    de
        call    ionGetPixel
        or      (hl)
        ld      (hl),a
        pop     de
        inc     d
        inc     e
        ld      a,d
        call    ionGetPixel
        or      (hl)
        ld      (hl),a
        call    videoCenterCopy
mapKeyLoop:
        call    waitKey
#ifdef TI86
        cp      GK_MORE
#else
        cp      GK_DEL
#endif
        jr      z,mapKeyLoop
        ret

intexBuyTwinMissile:
        ld      de,weaponTable+1
        ld      hl,intexWeaponsMenu
        ld      bc,1000
        jp      buyWeaponMap
intexBuyFlameArc:
        ld      de,weaponTable+2
        ld      hl,intexWeaponsMenu
        ld      bc,1700
        jp      buyWeaponMap
intexBuyPlasmaRifle:
        ld      de,weaponTable+3
        ld      hl,intexWeaponsMenu
        ld      bc,2500
        jr      buyWeaponMap
intexBuyRocketLauncher:
        ld      de,weaponTable+4
        ld      hl,intexWeaponsMenu
        ld      bc,3500
        jr      buyWeaponMap
intexBuyKeyPack:
        ld      hl,(credits)
        ld      bc,300
        sbc     hl,bc
        jp      c,intexMenuLoop
        ld      (credits),hl
        ld      a,(keyPasses)
        add     a,5
        ld      (keyPasses),a
        jp      intexMiscMenu
intexBuyAmmoClip:
        ld      hl,(credits)
        ld      bc,500
        sbc     hl,bc
        jp      c,intexMenuLoop
        ld      (credits),hl
        ld      hl,(ammo)
        ld      de,40
        add     hl,de
        ld      (ammo),hl
        jp      intexMiscMenu
intexBuyFirstAid:
        ld      hl,(credits)
        ld      bc,800
        sbc     hl,bc
        jp      c,intexMenuLoop
        ld      (credits),hl
        ld      hl,health
        ld      a,(hl)
        add     a,60
        ld      (hl),a
        cp      100
        jp      c,intexMiscMenu
        ld      (hl),100
        jp      intexMiscMenu
intexBuyRadarScanner:
        ld      de,radarScanner
        ld      hl,intexMiscMenu
        ld      bc,1500
        jr      buyWeaponMap
intexBuyExtraLife:
        ld      hl,(credits)
        ld      bc,3000
        sbc     hl,bc
        jp      c,intexMenuLoop
        ld      (credits),hl
        ld      hl,lives
        inc     (hl)
        jp      intexMiscMenu

;------------------------------------------------
; buyWeaponMap - Buy a Weapon or a Radar Scanner
;
; Input:    DE => Flag to check if already owned
;           HL => Where to jump to after purchase
;           BC = Cost of item
; Output:   None
;------------------------------------------------
buyWeaponMap:
        ld      a,(de)
        or      a
        jp      nz,intexMenuLoop
        push    hl
        ld      hl,(credits)
        sbc     hl,bc
        jr      c,notEnoughCredits
        ld      (credits),hl
        inc     a
        ld      (de),a
        pop     hl
        jp      (hl)
notEnoughCredits:
        pop     hl
        jp      intexMenuLoop

;------------------------------------------------
; intexExit - Exit Intex System
;
; Input:    None
; Output:   None
;------------------------------------------------
intexExit:
        bcall(_grbufclr)
        ld      de,5*256+0
        ld      hl,strExitIntex
        call    vPuts
        call    videoCenterCopy
        ld      b,50
#ifndef MIRAGE
        ei
        halt
        djnz    $-1
#else
        call    delayb
#endif
        jp      restoreVideo

.end
