;-----------------------------------------------------------;
;                                                           ;
; Alien Breed IV: Final Assault                             ;
; Small Screen Scrolling Routines                           ;
;                                                           ;
;-----------------------------------------------------------;

;------------------------------------------------
; scrollUp - Scroll the screen up a pixel
;
; Input:    None
; Output:   None
;------------------------------------------------
scrollUp:
        ld      hl,gbuf+12
        ld      de,gbuf
        ld      bc,63*12
        ldir
        ld      hl,gbuf+(63*12)
        ld      a,63
        ld      b,$34

;------------------------------------------------
; scrollVertical - Scroll screen vertically
;
; Input:    HL => Start of either top or bottom row of gbuf
;           A = {0, 63}
;           B = {inc (hl), dec (hl)}
; Output:   None
;------------------------------------------------
scrollVertical:
        ld      (_svA),a
        ld      a,b
        ld      (_svIncDec),a
        ld      (_svgbuf1),hl
        ld      (_svgbuf2),hl
        ld      de,11
        add     hl,de
        ld      (_svgbufEndRow),hl
        ld      de,-11
        add     hl,de
        ld      d,h
        ld      e,l
        inc     de
        ld      bc,11
        ld      (hl),0
        ldir
        ld      hl,scrY
        .db     $00
_svIncDec       = $-1
        ld      a,$00
_svA            = $-1
        add     a,(hl)
        ld      (tempY),a
        ld      a,(scrX)
        and     $07
        jr      nz,sh1
        ld      b,12
        xor     a
        ld      (tempX),a
        ld      (scrollBits),a
        jr      sh2
sh1:
        ld      b,11
        sub     8
        neg
        ld      (tempX),a
        ld      (scrollBits),a
sh2:
        ld      a,(tempY)
        and     $07
        ld      l,a
        ld      h,0
        ld      (scrollSkip),hl
        push    bc
        ld      hl,scrX
        ld      a,(tempX)
        add     a,(hl)
        ld      hl,(tempY)
        call    getTile
        pop     bc
sh3:
        push    bc
        push    hl
        ld      a,(hl)
        ld      c,0
        call    checkEnemyTriggers
        ld      l,(hl)
        ld      h,0
        TIMES8(hl)
        ld      de,startTiles
        add     hl,de
        ld      de,(scrollSkip)
        add     hl,de
        ld      a,(tempX)
        ld      de,$0000
_svgbuf1        = $-2
        call    putScrollByte
        pop     hl
        inc     hl
        ld      a,(tempX)
        add     a,8
        ld      (tempX),a
        pop     bc
        djnz    sh3
        ld      a,(scrollBits)
        or      a
        ret     z
        push    hl
        ld      b,a
        ld      l,(hl)
        ld      h,0
        TIMES8(hl)
        ld      de,startTiles
        add     hl,de
        ld      de,(scrollSkip)
        add     hl,de
        ld      a,(hl)
sh4:
        srl     a
        djnz    sh4
        ld      hl,$0000
_svgbufEndRow   = $-2
        or      (hl)
        ld      (hl),a
        pop     hl
        ld      de,-12
        add     hl,de
        ld      a,(scrollBits)
        sub     8
        neg
        ld      b,a
        ld      l,(hl)
        ld      h,0
        TIMES8(hl)
        ld      de,startTiles
        add     hl,de
        ld      de,(scrollSkip)
        add     hl,de
        ld      a,(hl)
sh5:
        add     a,a
        djnz    sh5
        ld      hl,$0000
_svgbuf2        = $-2
        or      (hl)
        ld      (hl),a
        ret

;------------------------------------------------
; scrollDown - Scroll the screen down a pixel
;
; Input:    None
; Output:   None
;------------------------------------------------
scrollDown:
        ld      hl,gbuf+755
        ld      de,gbuf+767
        ld      bc,63*12
        lddr
        ld      hl,gbuf
        xor     a
        ld      b,$35
        jp      scrollVertical

;------------------------------------------------
; scrollLeft - Scroll the screen left a pixel
;
; Input:    None
; Output:   None
;------------------------------------------------
scrollLeft:
        ld      hl,gbuf+(64*12)-1
        ld      c,64
sl1:
        ld      b,3
        or      a
sl2:
        rl      (hl)
        dec     hl
        rl      (hl)
        dec     hl
        rl      (hl)
        dec     hl
        rl      (hl)
        dec     hl
        djnz    sl2
        dec     c
        jr      nz,sl1
; Draw tiles at right of screen
        ld      hl,scrX
        inc     (hl)
        ld      a,95
        add     a,(hl)
        ld      (tempX),a
        and     $07
        ld      hl,pixelTable+1
        ld      e,a
        ld      d,0
        add     hl,de
        ld      a,(hl)
        ld      (scrollMask),a
        ld      a,(tempX)
        ld      hl,(scrY)
        call    getTile
        ld      (scrollPointer),hl
        ld      a,(scrY)
        and     $07
        ld      l,a
        ld      h,0
        ld      (scrollSkip),hl
        ld      (scrollBits),a
        sub     8
        neg
        ld      ix,gbuf+11
        ld      c,a
        ld      b,8
sl3:
        ld      hl,(scrollPointer)
        push    bc
        ld      a,(hl)
        ld      c,0
        call    checkEnemyTriggers
        pop     bc
        ld      l,(hl)
        ld      h,0
        TIMES8(hl)
        ld      de,startTiles
        add     hl,de
        ld      de,(scrollSkip)
        add     hl,de
        ld      de,12
sl4:
        ld      a,(scrollMask)
        and     (hl)
        jr      z,sl5
        set     0,(ix)
sl5:
        add     ix,de
        inc     hl
        dec     c
        jr      nz,sl4
        ld      hl,0
        ld      (scrollSkip),hl
        ld      hl,(scrollPointer)
        ld      de,(width)
        ld      d,0
        inc     e
        add     hl,de
        ld      (scrollPointer),hl
        ld      c,8
        djnz    sl3
        ld      a,(scrollBits)
        or      a
        ret     z
        ld      b,1
        ld      c,a
        xor     a
        ld      (scrollBits),a
        jp      sl3

;------------------------------------------------
; scrollRight - Scroll the screen right a pixel
;
; Input:    None
; Output:   None
;------------------------------------------------
scrollRight:
        ld      hl,gbuf
        ld      c,64
sr1:
        ld      b,3
        or      a
sr2:
        rr      (hl)
        inc     hl
        rr      (hl)
        inc     hl
        rr      (hl)
        inc     hl
        rr      (hl)
        inc     hl
        djnz    sr2
        dec     c
        jr      nz,sr1
; Draw tiles at left of screen
        ld      hl,scrX
        dec     (hl)
        ld      a,(hl)
        ld      (tempX),a
        and     $07
        ld      hl,pixelTable+1
        ld      e,a
        ld      d,0
        add     hl,de
        ld      a,(hl)
        ld      (scrollMask),a
        ld      a,(tempX)
        ld      hl,(scrY)
        call    getTile
        ld      (scrollPointer),hl
        ld      a,(scrY)
        and     $07
        ld      l,a
        ld      h,0
        ld      (scrollSkip),hl
        ld      (scrollBits),a
        sub     8
        neg
        ld      ix,gbuf
        ld      c,a
        ld      b,8
sr3:
        ld      hl,(scrollPointer)
        push    bc
        ld      a,(hl)
        ld      c,0
        call    checkEnemyTriggers
        pop     bc
        ld      l,(hl)
        ld      h,0
        TIMES8(hl)
        ld      de,startTiles
        add     hl,de
        ld      de,(scrollSkip) 
        add     hl,de
        ld      de,12
sr4:
        ld      a,(scrollMask)
        and     (hl)
        jr      z,sr5
        set     7,(ix)
sr5:
        add     ix,de
        inc     hl
        dec     c
        jr      nz,sr4
        ld      hl,0
        ld      (scrollSkip),hl
        ld      hl,(scrollPointer)
        ld      de,(width)
        ld      d,0
        inc     e
        add     hl,de
        ld      (scrollPointer),hl
        ld      c,8
        djnz    sr3
        ld      a,(scrollBits)
        or      a
        ret     z
        ld      b,1
        ld      c,a
        xor     a
        ld      (scrollBits),a
        jp      sr3

;------------------------------------------------
; putScrollByte - Put a byte to a row on the gbuf at a specified X coord
;
; Input:    A = X
;           HL => Byte
;           DE => Start of row on gbuf
; Output:   None
;------------------------------------------------
putScrollByte:
        ex      de,hl                           ; HL => Start of row on gbuf, DE => Byte
        ld      b,0
        ld      c,a
        and     $07
        ld      (_psb),a
        srl     c
        srl     c
        srl     c
        add     hl,bc
        ld      a,(de)
        ld      d,a
        ld      e,b
        ld      a,$00
_psb            = $-1
        or      a
        jr      z,psbSkip
psbLoop:
        srl     d
        rr      e
        dec     a
        jr      nz,psbLoop
psbSkip:
        ld      a,(hl)
        or      d
        ld      (hl),a
        inc     hl
        ld      a,(hl)
        or      e
        ld      (hl),a
        ret

.end
